﻿/*
Licensed under the MIT License

Copyright (c) 2010 Rasmus Wriedt Larsen
www.rasmuswriedtlarsen.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, and/or sublicense
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package
{
	import flash.display.MovieClip;
	import flash.display.Stage;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.ui.Mouse;
	import flash.ui.MouseCursor;
	
	/**
	 * ...
	 * @author Rasmus Wriedt Larsen
	 * For more information, check out http://gamedev.rasmuswriedtlarsen.com
	 */
	public class Hit extends MovieClip
	{
		public static var IDLE:String = "idle";
		public static var OVER:String = "over";
		public static var DOWN:String = "down";
		public static var PAUSE:String = "pause";
		
		private static var _isMouseDown:Boolean;
		
		private var _isPaused:Boolean;
		private var _isScaledDown:Boolean;
		
		private var _associatedMovieClip:MovieClip;
		private var _scaleUpDown:Number;
		private var _useDownFrameInstead:Boolean;
		private var _usePauseFrameInstead:Boolean;
		
		private var _lastAlpha:Number;
		
		private var idleFrame:int;
		private var overFrame:int;
		private var downFrame:int;
		private var pauseFrame:int;
		
		
		private static var _isActive:Boolean
		
		private static var _stageRef:Stage;
		
		// STATIC
		
		public static function init ( stageRef:Stage ) :void
		{
			if (!_stageRef )
			{
				_stageRef = stageRef;
				_stageRef.addEventListener ( MouseEvent.MOUSE_DOWN, onStageMouseDown,true ) ;
				_stageRef.addEventListener ( MouseEvent.MOUSE_DOWN, onStageMouseDown,false ) ;
				_stageRef.addEventListener ( MouseEvent.MOUSE_UP, onStageMouseUp,true ) ;
				_stageRef.addEventListener ( MouseEvent.MOUSE_UP, onStageMouseUp, false ) ;
				_stageRef.addEventListener ( Event.ACTIVATE, onActivate ) ;
				_stageRef.addEventListener ( Event.DEACTIVATE, onDeactivate ) ;
				
				_isActive = true;
			}
		}
		
		static private function onActivate(e:Event):void 
		{
			_isActive = true;
		}
		
		static private function onDeactivate(e:Event):void 
		{
			_isActive = false;
		}
		
		private static function onStageMouseDown(e:MouseEvent):void 
		{
			_isMouseDown = true;
		}
		
		private static function onStageMouseUp(e:MouseEvent):void 
		{
			_isMouseDown = false;
			Mouse.cursor = MouseCursor.AUTO;
		}
		
		// instance functions
		
		public function Hit() 
		{
			this.buttonMode = true;
			getChildAt ( 0 ) .alpha = 0;
			
			this.addEventListener ( MouseEvent.MOUSE_UP, onMouseUp, false, 0, true ) ;
			this.addEventListener ( MouseEvent.CLICK, onMouseClick, false, 0, true ) ;
			
		}
		
		// INTERACTIVITY
		
		public function addInteractivity ( mc:MovieClip , scaleUpDown:Number = 0.15, useDownFrameInstead:Boolean = false ) :void
		{
			_scaleUpDown = 1 - scaleUpDown;
			_associatedMovieClip = mc;
			_useDownFrameInstead = useDownFrameInstead;
			
			_associatedMovieClip.gotoAndStop ( IDLE ) ;
			idleFrame = _associatedMovieClip.currentFrame;
			
			var frameBefore:int = _associatedMovieClip.currentFrame;
			_associatedMovieClip.gotoAndStop ( OVER ) ;
			overFrame = _associatedMovieClip.currentFrame;
			if ( overFrame == frameBefore ) overFrame = 2;
			
			frameBefore = _associatedMovieClip.currentFrame;
			_associatedMovieClip.gotoAndStop ( DOWN ) ;
			downFrame = _associatedMovieClip.currentFrame;
			if ( downFrame == frameBefore ) downFrame = 3;
			
			frameBefore = _associatedMovieClip.currentFrame;
			_associatedMovieClip.gotoAndStop ( PAUSE ) ;
			pauseFrame = _associatedMovieClip.currentFrame;
			if ( pauseFrame == frameBefore ) pauseFrame = 4;
			
			_associatedMovieClip.gotoAndStop ( idleFrame ) ;
			
			this.addEventListener ( MouseEvent.MOUSE_OVER, onMouseOver , false, 0, true ) ;
			this.addEventListener ( MouseEvent.MOUSE_OUT, onMouseOut , false, 0, true ) ;
			
			this.addEventListener ( MouseEvent.MOUSE_DOWN, onMouseDown, false, 0, true ) ;
			
			
			_isScaledDown = false;
			
			if ( checkIfMouseIsOver ( ) )
			{
				addOverEffect ( ) ;
			}
		}
		
		public function removeInteractivity ( ) :void
		{
			if ( _associatedMovieClip )
			{
				this.removeEventListener ( MouseEvent.MOUSE_OVER, onMouseOver ) ;
				this.removeEventListener ( MouseEvent.MOUSE_OUT, onMouseOut  ) ;
				
				this.removeEventListener ( MouseEvent.MOUSE_DOWN, onMouseDown ) ;
				this.removeEventListener ( MouseEvent.MOUSE_UP, onMouseUp ) ;
				
				this.removeEventListener ( MouseEvent.CLICK, onMouseClick ) ;
				
				_associatedMovieClip = null;
			}
		}
		
		public function pauseInteractivity ( newAlpha:Number = 1, usePauseFrameInstead:Boolean = false ) :void
		{
			_isPaused = true;
			this.buttonMode = false;
			
			_usePauseFrameInstead = usePauseFrameInstead;
			
			if ( _associatedMovieClip )
			{
				if ( checkIfMouseIsOver ( ) )
				{
					removeDownEffect ( ) ;
					removeOverEffect ( ) ;
				}
				
				if (!_usePauseFrameInstead )
				{
					_lastAlpha = _associatedMovieClip.alpha;
					_associatedMovieClip.alpha = newAlpha;
				}
				else
				{
					_associatedMovieClip.gotoAndStop ( pauseFrame ) ;
				}
			}
			else if ( checkIfMouseIsOver ( ) )
			{
				Mouse.cursor = MouseCursor.AUTO;
			}
		}
		
		public function unpauseInteractivity ( ) :void
		{
			_isPaused = false;
			this.buttonMode = true;
			
			if ( _associatedMovieClip )
			{
				if (!_usePauseFrameInstead )
				{
					_associatedMovieClip.alpha = _lastAlpha;
				}
				if ( checkIfMouseIsOver ( ) )
				{
					addOverEffect ( ) ;
				}
				else
				{
					_associatedMovieClip.gotoAndStop ( idleFrame ) ;
				}
			}
			else if ( checkIfMouseIsOver ( ) )
			{
				Mouse.cursor = MouseCursor.AUTO;
			}
		}
		
		public function get isInteractive ( ) :Boolean
		{
			return _isPaused ? false : true;
		}
		
		private function checkIfMouseIsOver ( ) :Boolean
		{
			if ( !_isActive ) { return false; }
			
			if ( this.hitTestPoint ( _stageRef.mouseX, _stageRef.mouseY, true ) ) 
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		
		// MOUSE HANDELERS
		
		private function onMouseOver(e:MouseEvent):void 
		{
			if ( _isPaused ) { return; }
			addOverEffect ( ) ;
		}
		
		private function onMouseOut(e:MouseEvent):void 
		{
			if ( _isPaused ) { return; }
			removeOverEffect ( ) ;
		}
		
		private function onMouseDown(e:MouseEvent):void 
		{
			if ( _isPaused ) { return; }
			addDownEffect ( ) ;
		}
		
		private function onMouseUp(e:MouseEvent):void 
		{
			if ( _isPaused ) { e.stopImmediatePropagation ( ) ;  return; }
			if ( _associatedMovieClip )
			{
				removeDownEffect ( ) ;
			}
		}
		
		private function onMouseClick(e:MouseEvent):void 
		{
			if ( _isPaused ) { e.stopImmediatePropagation ( ) ; }
		}
		
		// EFFECTS
		
		private function addDownEffect():void
		{
			if ( !_useDownFrameInstead && !_isScaledDown)
			{
				_associatedMovieClip.scaleX = _associatedMovieClip.scaleX * _scaleUpDown;
				_associatedMovieClip.scaleY = _associatedMovieClip.scaleY * _scaleUpDown;
				_isScaledDown = true;
			}
			else
			{
				_associatedMovieClip.gotoAndStop ( downFrame ) ;
			}
		}
		
		private function removeDownEffect():void
		{
			if ( !_useDownFrameInstead && _isScaledDown)
			{
				_associatedMovieClip.scaleX = _associatedMovieClip.scaleX / _scaleUpDown;
				_associatedMovieClip.scaleY = _associatedMovieClip.scaleY / _scaleUpDown;
				_isScaledDown = false;
			}
			else
			{
				_associatedMovieClip.gotoAndStop ( overFrame ) ;
			}
		}
		
		private function addOverEffect():void
		{
			_associatedMovieClip.gotoAndStop ( overFrame ) ;
			Mouse.cursor = MouseCursor.BUTTON;
			if ( _isMouseDown )
			{
				addDownEffect ( ) ;
			}
		}
		
		private function removeOverEffect():void
		{
			Mouse.cursor = MouseCursor.ARROW;
			if ( _isMouseDown )
			{
				removeDownEffect ( ) ;
			}
			_associatedMovieClip.gotoAndStop ( idleFrame ) ;
		}
		
	}

}